from django.http import HttpResponse
from django.test import RequestFactory
from django.test.utils import override_settings
from django.core.exceptions import ImproperlyConfigured

from django_hosts.middleware import (HostsRequestMiddleware,
                                     HostsResponseMiddleware)

from .base import HostsTestCase


class MiddlewareTests(HostsTestCase):

    def test_missing_hostconf_setting(self):
        self.assertRaisesMessage(ImproperlyConfigured,
            'Missing ROOT_HOSTCONF setting', HostsRequestMiddleware)

    @override_settings(ROOT_HOSTCONF='tests.hosts.simple')
    def test_missing_default_hosts(self):
        self.assertRaisesMessage(ImproperlyConfigured,
            'Missing DEFAULT_HOST setting', HostsRequestMiddleware)

    @override_settings(
        ROOT_HOSTCONF='tests.hosts.simple',
        DEFAULT_HOST='boo')
    def test_wrong_default_hosts(self):
        self.assertRaisesMessage(ImproperlyConfigured,
            "Invalid DEFAULT_HOST setting: No host called 'boo' exists",
            HostsRequestMiddleware)

    @override_settings(
        ROOT_HOSTCONF='tests.hosts.simple',
        DEFAULT_HOST='www')
    def test_request_urlconf_module(self):
        rf = RequestFactory(HTTP_HOST='other.example.com')
        request = rf.get('/simple/')
        middleware = HostsRequestMiddleware()
        middleware.process_request(request)
        self.assertEqual(request.urlconf, 'tests.urls.simple')

    @override_settings(
        ROOT_HOSTCONF='tests.hosts.simple',
        DEFAULT_HOST='www')
    def test_response_urlconf_module(self):
        rf = RequestFactory(HTTP_HOST='other.example.com')
        request = rf.get('/simple/')
        middleware = HostsResponseMiddleware()
        middleware.process_response(request, HttpResponse('test'))
        self.assertEqual(request.urlconf, 'tests.urls.simple')

    @override_settings(
        ROOT_HOSTCONF='tests.hosts.simple',
        DEFAULT_HOST='with_view_kwargs')
    def test_fallback_to_defaulthost(self):
        rf = RequestFactory(HTTP_HOST='ss.example.com')
        request = rf.get('/template/test/')
        middleware = HostsRequestMiddleware()
        middleware.process_request(request)
        self.assertEqual(request.urlconf, 'tests.urls.complex')
        host, kwargs = middleware.get_host('non-existing')
        self.assertEqual(host.name, 'with_view_kwargs')

    @override_settings(
        ROOT_HOSTCONF='tests.hosts.simple',
        DEFAULT_HOST='www',
        ALLOWED_HOSTS=['somehost.com'],
        DEBUG=False,
        MIDDLEWARE_CLASSES=[
            'django_hosts.middleware.HostsRequestMiddleware',
            'django_hosts.middleware.HostsResponseMiddleware',
        ])
    def test_fallback_with_evil_host(self):
        response = self.client.get('/', HTTP_HOST='evil.com')
        self.assertEqual(response.status_code, 400)

    @override_settings(
        ROOT_HOSTCONF='tests.hosts.multiple',
        DEFAULT_HOST='multiple')
    def test_multiple_subdomains(self):
        rf = RequestFactory(HTTP_HOST='spam.eggs.example.com')
        request = rf.get('/multiple/')
        middleware = HostsRequestMiddleware()
        middleware.process_request(request)
        self.assertEqual(request.urlconf, 'tests.urls.multiple')

    @override_settings(
        MIDDLEWARE_CLASSES=['debug_toolbar.middleware.DebugToolbarMiddleware',
                            'django_hosts.middleware.HostsRequestMiddleware'],
        ROOT_HOSTCONF='tests.hosts.multiple',
        DEFAULT_HOST='multiple')
    def test_debug_toolbar_new_warning(self):
        self.assertRaises(ImproperlyConfigured, HostsRequestMiddleware)

    @override_settings(
        MIDDLEWARE_CLASSES=['debug_toolbar.middleware.DebugToolbarMiddleware',
                            'django_hosts.middleware.HostsMiddleware'],
        ROOT_HOSTCONF='tests.hosts.multiple',
        DEFAULT_HOST='multiple')
    def test_debug_toolbar_old_warning(self):
        self.assertRaises(ImproperlyConfigured, HostsRequestMiddleware)



from django.core.exceptions import ImproperlyConfigured
from django_hosts.defaults import host, patterns
from django_hosts.resolvers import get_host_patterns

from .base import HostsTestCase


class PatternsTests(HostsTestCase):

    def test_pattern(self):
        host_patterns = patterns('',
            host(r'api', 'api.urls', name='api'),
        )
        self.assertEqual(len(host_patterns), 1)
        self.assertTrue(isinstance(host_patterns[0], host))
        self.assertEqual(repr(host_patterns[0]),
                         "<host api: regex='api' urlconf='api.urls' "
                         "scheme='//' port=''>")

    def test_pattern_as_tuple(self):
        host_patterns = patterns('',
            (r'api', 'api.urls', 'api'),
        )
        self.assertEqual(len(host_patterns), 1)
        self.assertTrue(isinstance(host_patterns[0], host))

    def test_pattern_with_duplicate(self):
        api_host = host(r'api', 'api.urls', name='api')
        self.assertRaises(ImproperlyConfigured,
                          patterns, '', api_host, api_host)

    def test_pattern_with_prefix(self):
        host_patterns = patterns('mysite',
            host(r'api', 'api.urls', name='api'),
        )
        self.assertEqual(len(host_patterns), 1)
        self.assertTrue(isinstance(host_patterns[0], host))
        self.assertEqual(host_patterns[0].urlconf, 'mysite.api.urls')


class HostTests(HostsTestCase):

    def test_host(self):
        api_host = host(r'api', 'api.urls', name='api')
        self.assertTrue(isinstance(api_host, host))

    def test_host_prefix(self):
        api_host = host(r'api', 'api.urls', name='api', prefix='spam.eggs')
        self.assertEqual(api_host.urlconf, 'spam.eggs.api.urls')

    def test_host_string_callback(self):
        api_host = host(r'api', 'api.urls', name='api',
                        callback='django_hosts.resolvers.get_host_patterns')
        self.assertEqual(api_host.callback, get_host_patterns)

    def test_host_callable_callback(self):
        api_host = host(r'api', 'api.urls', name='api',
                        callback=get_host_patterns)
        self.assertEqual(api_host.callback, get_host_patterns)

    def test_host_nonexistent_callback(self):
        api_host = host(r'api', 'api.urls', name='api',
                        callback='whatever.non_existent')
        self.assertRaisesMessageIn(ImproperlyConfigured,
            "Could not import 'whatever'. Error was: No module named",
            lambda: api_host.callback)

        api_host = host(r'api', 'api.urls', name='api',
                        callback='django_hosts.non_existent')
        try:
            self.assertRaisesMessageIn(ImproperlyConfigured,
                "Could not import 'django_hosts.non_existent'. "
                "Callable does not exist in module",
                lambda: api_host.callback)
        except AssertionError:
            self.assertRaisesMessageIn(ImproperlyConfigured,
                "Could not import django_hosts.non_existent. "
                "Callable does not exist in module",
                lambda: api_host.callback)

        api_host = host(r'api', 'api.urls', name='api',
                        callback='tests.broken_module.yeah_yeah')
        self.assertRaises(ImproperlyConfigured, lambda: api_host.callback)



from django.core.exceptions import ImproperlyConfigured
from django.core.urlresolvers import NoReverseMatch
from django.test.utils import override_settings

from django_hosts.resolvers import (get_hostconf_module, get_host_patterns,
                                    get_host, reverse_host, reverse)

from .base import HostsTestCase
from .hosts import simple


class ReverseTest(HostsTestCase):

    @override_settings(ROOT_HOSTCONF='tests.hosts.simple')
    def test_reverse_host(self):
        self.assertRaises(ValueError,
            reverse_host, 'with_kwargs', ['spam'], dict(eggs='spam'))
        self.assertRaises(NoReverseMatch,
            reverse_host, 'with_kwargs', ['spam', 'eggs'])
        self.assertRaises(NoReverseMatch,
            reverse_host, 'with_kwargs', [], dict(eggs='spam', spam='eggs'))
        self.assertEqual('johndoe',
            reverse_host('with_kwargs', None, dict(username='johndoe')))
        self.assertEqual(reverse_host('with_args', ['johndoe']), 'johndoe')
        with self.settings(PARENT_HOST='spam.eggs'):
            self.assertEqual(reverse_host('with_args', ['johndoe']),
                             'johndoe.spam.eggs')

    @override_settings(
        ROOT_HOSTCONF='tests.hosts.simple',
        PARENT_HOST='spam.eggs')
    def test_reverse(self):
        self.assertEqual(reverse('simple-direct', host='static'),
                         '//static.spam.eggs/simple/')

    @override_settings(
        ROOT_HOSTCONF='tests.hosts.simple',
        PARENT_HOST='example.com')
    def test_reverse_without_www(self):
        self.assertEqual(reverse('simple-direct', host='without_www'),
                         '//example.com/simple/')

    @override_settings(
        ROOT_HOSTCONF='tests.hosts.blank',
        PARENT_HOST='example.com')
    def test_reverse_blank(self):
        self.assertEqual(reverse('simple-direct', host='blank_or_www'),
                         '//example.com/simple/')

    @override_settings(
        ROOT_HOSTCONF='tests.hosts.simple',
        PARENT_HOST='spam.eggs')
    def test_reverse_custom_scheme(self):
        self.assertEqual(reverse('simple-direct', host='scheme'),
                         'https://scheme.spam.eggs/simple/')
        self.assertEqual(reverse('simple-direct', host='scheme', scheme='ftp'),
                         'ftp://scheme.spam.eggs/simple/')

    @override_settings(
        ROOT_HOSTCONF='tests.hosts.simple',
        PARENT_HOST='spam.eggs')
    def test_reverse_custom_port(self):
        self.assertEqual(reverse('simple-direct', host='port'),
                         '//port.spam.eggs:12345/simple/')
        self.assertEqual(reverse('simple-direct', host='port', port='1337'),
                         '//port.spam.eggs:1337/simple/')


class UtilityTests(HostsTestCase):

    @override_settings(ROOT_HOSTCONF='tests.hosts.simple')
    def test_get_hostconf_module(self):
        self.assertEqual(get_hostconf_module(), simple)

    def test_get_hostconf_module_no_default(self):
        self.assertEqual(
            get_hostconf_module('tests.hosts.simple'), simple)

    def test_missing_host_patterns(self):
        self.assertRaisesMessage(ImproperlyConfigured,
            'Missing ROOT_HOSTCONF setting', get_host_patterns)

    @override_settings(ROOT_HOSTCONF='tests.hosts')
    def test_missing_host_patterns_in_module(self):
        self.assertRaisesMessage(ImproperlyConfigured,
            "Missing host_patterns in 'tests.hosts'",
            get_host_patterns)

    @override_settings(ROOT_HOSTCONF='tests.hosts.simple')
    def test_get_working_host_patterns(self):
        self.assertEqual(get_host_patterns(), simple.host_patterns)

    @override_settings(ROOT_HOSTCONF='tests.hosts.simple')
    def test_get_host(self):
        self.assertEqual(get_host('static').name, 'static')
        self.assertRaisesMessage(NoReverseMatch,
            "No host called 'non-existent' exists", get_host, 'non-existent')

    @override_settings(ROOT_HOSTCONF='tests.hosts.appended')
    def test_appended_patterns(self):
        self.assertEqual(get_host('special').name, 'special')



from django_hosts.utils import normalize_scheme, normalize_port

from .base import HostsTestCase


class UtilsTest(HostsTestCase):

    def test_normalize_scheme(self):
        self.assertEqual(normalize_scheme('http'), 'http://')
        self.assertEqual(normalize_scheme('http:'), 'http://')
        self.assertEqual(normalize_scheme(), '//')

    def test_normalize_port(self):
        self.assertEqual(normalize_port(None), '')
        self.assertEqual(normalize_port(':80:'), ':80')
        self.assertEqual(normalize_port('80'), ':80')
        self.assertEqual(normalize_port('80:'), ':80')
        self.assertEqual(normalize_port(), '')



"""
A template tag library that is automatically added to Django's
built-in template tags if the setting HOST_OVERRIDE_URL_TAG is set
to True.
"""
from django import template

from .hosts import host_url

register = template.Library()


@register.tag
def url(parser, token):
    """
    A tag to override the built-in url template tag. Accepts host parameters
    optionally.

    {% url 'view-name' host 'host-name'  %}
    {% url 'view-name' host 'host-name' 'spam' %}
    {% url 'view-name' host 'host-name' scheme 'https' %}
    {% url 'view-name' host 'host-name' as url_on_host_variable %}
    {% url 'view-name' varg1=vvalue1 host 'host-name' 'spam' 'hvalue1' %}
    {% url 'view-name' vvalue2 host 'host-name' 'spam' harg2=hvalue2 %}
    """
    return host_url(parser, token)




def normalize_scheme(scheme=None, default='//'):
    if scheme is None:
        scheme = default
    elif scheme.endswith(':'):
        scheme = '%s//' % scheme
    elif '//' not in scheme:
        scheme = '%s://' % scheme
    return scheme


def normalize_port(port=None):
    if port is None:
        port = ''
    elif ':' in port:
        port = ':%s' % port.strip(':')
    elif port:
        port = ':%s' % port
    return port



from django.apps import AppConfig
from django.conf import settings
from django.core import checks
from django.core.exceptions import ImproperlyConfigured
try:
    from django.template.base import add_to_builtins
except ImportError:  # Django 1.9+
    add_to_builtins = None
from django.utils.translation import ugettext_lazy as _

from .checks import check_default_host, check_root_hostconf


class HostsConfig(AppConfig):  # pragma: no cover
    """
    The django-hosts app config that conditionally adds its url to the
    built-ins of Django if the HOST_OVERRIDE_URL_TAG setting is set.
    """
    name = 'django_hosts'
    verbose_name = _('Hosts')

    def ready(self):
        checks.register(check_root_hostconf)
        checks.register(check_default_host)

        if getattr(settings, 'HOST_OVERRIDE_URL_TAG', False):
            if add_to_builtins:
                add_to_builtins('django_hosts.templatetags.hosts_override')
            else:
                raise ImproperlyConfigured(
                    "On Django 1.9+, you must add "
                    "'django_hosts.templatetags.hosts_override' to the "
                    "TEMPLATES['OPTIONS']['builtins'] list instead of using "
                    "the HOST_OVERRIDE_URL_TAG setting."
                )



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured, DisallowedHost
from django.core.urlresolvers import NoReverseMatch, set_urlconf, get_urlconf

from .resolvers import get_host_patterns, get_host


class HostsBaseMiddleware(object):
    """
    Adjust incoming request's urlconf based on hosts defined in
    settings.ROOT_HOSTCONF module.
    """
    old_hosts_middleware = 'django_hosts.middleware.HostsMiddleware'
    new_hosts_middleware = 'django_hosts.middleware.HostsRequestMiddleware'
    toolbar_middleware = 'debug_toolbar.middleware.DebugToolbarMiddleware'

    def __init__(self):
        self.current_urlconf = None
        self.host_patterns = get_host_patterns()
        try:
            self.default_host = get_host()
        except NoReverseMatch as exc:
            raise ImproperlyConfigured("Invalid DEFAULT_HOST setting: %s" %
                                       exc)

        middlewares = list(settings.MIDDLEWARE_CLASSES)

        show_exception = False
        if (self.old_hosts_middleware in middlewares and
                self.toolbar_middleware in middlewares):
            show_exception = (middlewares.index(self.old_hosts_middleware) >
                              middlewares.index(self.toolbar_middleware))

        if not show_exception and (self.new_hosts_middleware in middlewares and
                                   self.toolbar_middleware in middlewares):
            show_exception = (middlewares.index(self.new_hosts_middleware) >
                              middlewares.index(self.toolbar_middleware))

        if show_exception:
            raise ImproperlyConfigured(
                "The django-hosts and django-debug-toolbar middlewares "
                "are in the wrong order. Make sure the django-hosts "
                "middleware comes before the django-debug-toolbar "
                "middleware in the MIDDLEWARE_CLASSES setting.")

    def get_host(self, request_host):
        for host in self.host_patterns:
            match = host.compiled_regex.match(request_host)
            if match:
                return host, match.groupdict()
        return self.default_host, {}


class HostsRequestMiddleware(HostsBaseMiddleware):
    def process_request(self, request):
        # Find best match, falling back to settings.DEFAULT_HOST
        host, kwargs = self.get_host(request.get_host())
        # This is the main part of this middleware
        request.urlconf = host.urlconf
        request.host = host
        # But we have to temporarily override the URLconf
        # already to allow correctly reversing host URLs in
        # the host callback, if needed.
        current_urlconf = get_urlconf()
        try:
            set_urlconf(host.urlconf)
            return host.callback(request, **kwargs)
        finally:
            # Reset URLconf for this thread on the way out for complete
            # isolation of request.urlconf
            set_urlconf(current_urlconf)


class HostsResponseMiddleware(HostsBaseMiddleware):
    def process_response(self, request, response):
        # Django resets the base urlconf when it starts to process
        # the response, so we need to set this again, in case
        # any of our middleware makes use of host, etc URLs.

        # Find best match, falling back to settings.DEFAULT_HOST
        try:
            host, kwargs = self.get_host(request.get_host())
        except DisallowedHost:
            # Bail out early, there is nothing to reset as HostsRequestMiddleware
            # never gets called with an invalid host.
            return response
        # This is the main part of this middleware
        request.urlconf = host.urlconf
        request.host = host

        set_urlconf(host.urlconf)
        return response


class HostsMiddleware(HostsRequestMiddleware):  # pragma: no cover
    """
    Provided for backwards-compatibility.
    """
    def __init__(self):
        import warnings
        warnings.warn("The 'django_hosts.middleware.HostsMiddleware' "
                      "middleware has been split into HostsRequestMiddleware "
                      "and HostsResponseMiddleware. Please consult the "
                      "documentation and update your middleware settings.",
                      PendingDeprecationWarning)
        super(HostsMiddleware, self).__init__()



from django.conf import settings
from django.core import checks

E001 = checks.Error(
    "Missing 'DEFAULT_HOST' setting.",
    hint="Has to be the name of the default host pattern.",
    id='django_hosts.E001',
)

E002 = checks.Error(
    "Missing 'ROOT_HOSTCONF' setting.",
    hint="Has to be the dotted Python import path of "
         "the module containing your host patterns.",
    id='django_hosts.E002',
)


def check_default_host(app_configs, **kwargs):  # pragma: no cover
    return [] if getattr(settings, 'DEFAULT_HOST', False) else [E001]


def check_root_hostconf(app_configs, **kwargs):  # pragma: no cover
    return [] if getattr(settings, 'ROOT_HOSTCONF', False) else [E002]



from django.conf import settings
from django.shortcuts import get_object_or_404
from django.utils.functional import LazyObject

from .resolvers import reverse_host

HOST_SITE_TIMEOUT = getattr(settings, 'HOST_SITE_TIMEOUT', 3600)


class LazySite(LazyObject):

    def __init__(self, request, *args, **kwargs):
        super(LazySite, self).__init__()
        self.__dict__.update({
            'name': request.host.name,
            'args': args,
            'kwargs': kwargs,
        })

    def _setup(self):
        host = reverse_host(self.name, args=self.args, kwargs=self.kwargs)
        from django.contrib.sites.models import Site
        site = get_object_or_404(Site, domain__iexact=host)
        self._wrapped = site


class CachedLazySite(LazySite):

    def _setup(self):
        host = reverse_host(self.name, args=self.args, kwargs=self.kwargs)
        cache_key = "hosts:%s" % host
        from django.core.cache import cache
        site = cache.get(cache_key, None)
        if site is not None:
            self._wrapped = site
            return
        from django.contrib.sites.models import Site
        site = get_object_or_404(Site, domain__iexact=host)
        cache.set(cache_key, site, HOST_SITE_TIMEOUT)
        self._wrapped = site


def host_site(request, *args, **kwargs):
    """
    A callback function which uses the :mod:`django.contrib.sites` contrib
    app included in Django to match a host to a
    :class:`~django.contrib.sites.models.Site` instance, setting a
    ``request.site`` attribute on success.

    :param request: the request object passed from the middleware
    :param \*args: the parameters as matched by the host patterns
    :param \*\*kwargs: the keyed parameters as matched by the host patterns

    It's important to note that this uses
    :func:`~django_hosts.resolvers.reverse_host` behind the scenes to
    reverse the host with the given arguments and keyed arguments to
    enable a flexible configuration of what will be used to retrieve
    the :class:`~django.contrib.sites.models.Site` instance -- in the end
    the callback will use a ``domain__iexact`` lookup to get it.

    For example, imagine a host conf with a username parameter::

        from django.conf import settings
        from django_hosts import patterns, host

        settings.PARENT_HOST = 'example.com'

        host_patterns = patterns('',
            host(r'www', settings.ROOT_URLCONF, name='www'),
            host(r'(?P<username>\w+)', 'path.to.custom_urls',
                 callback='django_hosts.callbacks.host_site',
                 name='user-sites'),
        )

    When requesting this website with the host ``jezdez.example.com``,
    the callback will act as if you'd do::

        request.site = Site.objects.get(domain__iexact='jezdez.example.com')

    ..since the result of calling :func:`~django_hosts.resolvers.reverse_host`
    with the username ``'jezdez'`` is ``'jezdez.example.com'``.

    Later, in your views, you can nicely refer to the current site
    as ``request.site`` for further site-specific functionality.
    """
    request.site = LazySite(request, *args, **kwargs)


def cached_host_site(request, *args, **kwargs):
    """
    A callback function similar to :func:`~django_hosts.callbacks.host_site`
    which caches the resulting :class:`~django.contrib.sites.models.Site`
    instance in the default cache backend for the time specfified as
    :attr:`~django.conf.settings.HOST_SITE_TIMEOUT`.

    :param request: the request object passed from the middleware
    :param \*args: the parameters as matched by the host patterns
    :param \*\*kwargs: the keyed parameters as matched by the host patterns
    """
    request.site = CachedLazySite(request, *args, **kwargs)



import re
import warnings

from django import template
from django.conf import settings
from django.template import TemplateSyntaxError
from django.utils import six
from django.template.base import FilterExpression
from django.template.defaulttags import URLNode
from django.utils.encoding import iri_to_uri, smart_str
from django.core.urlresolvers import set_urlconf, get_urlconf

from ..resolvers import reverse_host, get_host
from ..utils import normalize_scheme, normalize_port

register = template.Library()

kwarg_re = re.compile(r"(?:(\w+)=)?(.+)")


class HostURLNode(URLNode):

    def __init__(self, *args, **kwargs):
        self.host = kwargs.pop('host')
        self.host_args = kwargs.pop('host_args')
        self.host_kwargs = kwargs.pop('host_kwargs')
        self.scheme = kwargs.pop('scheme')
        self.port = kwargs.pop('port')
        super(HostURLNode, self).__init__(*args, **kwargs)

    def maybe_resolve(self, var, context):
        """
        Variable may have already been resolved
        in e.g. a LoopNode, so we only resolve()
        if needed.
        """
        if isinstance(var, FilterExpression):
            return var.resolve(context)
        return var

    def render(self, context):
        host = get_host(self.maybe_resolve(self.host, context))
        current_urlconf = get_urlconf()
        try:
            set_urlconf(host.urlconf)
            path = super(HostURLNode, self).render(context)
            if self.asvar:
                path = context[self.asvar]
        finally:
            set_urlconf(current_urlconf)

        host_args = [self.maybe_resolve(x, context) for x in self.host_args]

        host_kwargs = dict((smart_str(k, 'ascii'),
                            self.maybe_resolve(v, context))
                           for k, v in six.iteritems(self.host_kwargs))

        if self.scheme:
            scheme = normalize_scheme(self.maybe_resolve(self.scheme, context))
        else:
            scheme = host.scheme

        if self.port:
            port = normalize_port(self.maybe_resolve(self.port, context))
        else:
            port = host.port

        hostname = reverse_host(host, args=host_args, kwargs=host_kwargs)

        uri = iri_to_uri('%s%s%s%s' % (scheme, hostname, port, path))

        if self.asvar:
            context[self.asvar] = uri
            return ''
        else:
            return uri


def parse_params(name, parser, bits):
    args = []
    kwargs = {}
    for bit in bits:
        match = kwarg_re.match(bit)
        if not match:
            raise TemplateSyntaxError("Malformed arguments to %s tag" % name)
        name, value = match.groups()
        if name:
            kwargs[name] = parser.compile_filter(value)
        else:
            args.append(parser.compile_filter(value))
    return args, kwargs


def fetch_arg(name, arg, bits, consume=True):
    try:
        pivot = bits.index(arg)
        try:
            value = bits[pivot + 1]
        except IndexError:
            raise TemplateSyntaxError("'%s' arguments must include "
                                      "a variable name after '%s'" %
                                      (name, arg))
        else:
            if consume:
                del bits[pivot:pivot + 2]
            return value, pivot, bits
    except ValueError:
        return None, None, bits


@register.tag
def host_url(parser, token):
    """
    Simple tag to reverse the URL inclusing a host.

    {% host_url 'view-name' host 'host-name'  %}
    {% host_url 'view-name' host 'host-name' 'spam' %}
    {% host_url 'view-name' host 'host-name' scheme 'https' %}
    {% host_url 'view-name' host 'host-name' as url_on_host_variable %}
    {% host_url 'view-name' varg1=vvalue1 host 'host-name' 'spam' 'hvalue1' %}
    {% host_url 'view-name' vvalue2 host 'host-name' 'spam' harg2=hvalue2 %}
    """
    bits = token.split_contents()
    name = bits[0]
    if len(bits) < 2:
        raise TemplateSyntaxError("'%s' takes at least one argument"
                                  " (path to a view)" % name)

    try:
        view_name = parser.compile_filter(bits[1])
    except TemplateSyntaxError as exc:
        exc.args = (exc.args[0] + ". "
                    "The syntax of the 'url' template tag has changed in "
                    "Django 1.5, see the docs. The view name is now "
                    "quoted unless it's meant as a variable."),
        raise

    asvar, pivot, bits = fetch_arg(name, 'as', bits[1:])  # Strip off viewname
    scheme, pivot, bits = fetch_arg(name, 'scheme', bits)
    if scheme:
        scheme = parser.compile_filter(scheme)
    port, pivot, bits = fetch_arg(name, 'port', bits)
    if port:
        port = parser.compile_filter(port)

    host, pivot, bits = fetch_arg(name, 'host', bits, consume=False)

    if host is None:
        host, pivot, bits = fetch_arg(name, 'on', bits, consume=False)
        warnings.warn("The 'on' keyword of the '%s' template tag is pending "
                      "deprecation in favor of the 'host' keyword. Please "
                      "upgrade your templates accordingly.",
                      PendingDeprecationWarning)

    if host:
        host = parser.compile_filter(host)
        view_args, view_kwargs = parse_params(name, parser, bits[1:pivot])
        host_args, host_kwargs = parse_params(name, parser, bits[pivot + 2:])
    else:
        # No host was given so use the default host
        host = settings.DEFAULT_HOST
        view_args, view_kwargs = parse_params(name, parser, bits[1:])
        host_args, host_kwargs = (), {}

    return HostURLNode(view_name=view_name, args=view_args, kwargs=view_kwargs,
                       asvar=asvar, host=host, host_args=host_args,
                       host_kwargs=host_kwargs, scheme=scheme, port=port)



"""
When defining hostconfs you need to use the ``patterns`` and ``host`` helpers
"""
import re
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured, ViewDoesNotExist
from django.core.urlresolvers import (get_mod_func,
                                      get_callable as actual_get_callable)
from django.utils.encoding import smart_str
from django.utils.functional import cached_property

from .utils import normalize_scheme, normalize_port

_callable_cache = {}  # Maps view and url pattern names to their view functions


def get_callable(lookup_view, can_fail=False):
    """
    Convert a string version of a function name to the callable object.

    If the lookup_view is not an import path, it is assumed to be a URL pattern
    label and the original string is returned.

    If can_fail is True, lookup_view might be a URL pattern label, so errors
    during the import fail and the string is returned.
    """
    try:
        return actual_get_callable(lookup_view, can_fail)
    except ViewDoesNotExist as exc:
        raise ImproperlyConfigured(exc.args[0].replace('View', 'Callable'))


def patterns(prefix, *args):
    """
    The function to define the list of hosts (aka hostconfs), e.g.::

        from django_hosts import patterns

        host_patterns = patterns('path.to',
            (r'www', 'urls.default', 'default'),
            (r'api', 'urls.api', 'api'),
        )

    :param prefix: the URLconf prefix to pass to the host object
    :type prefix: str
    :param \*args: a list of :class:`~django_hosts.defaults.hosts` instances
                   or an iterable thereof
    """
    hosts = []
    for arg in args:
        if isinstance(arg, (list, tuple)):
            arg = host(prefix=prefix, *arg)
        else:
            arg.add_prefix(prefix)
        name = arg.name
        if name in [h.name for h in hosts]:
            raise ImproperlyConfigured("Duplicate host name: %s" % name)
        hosts.append(arg)
    return hosts


class host(object):
    """
    The host object used in host conf together with the
    :func:`django_hosts.defaults.patterns` function, e.g.::

        from django_hosts import patterns, host

        host_patterns = patterns('path.to',
            host(r'www', 'urls.default', name='default'),
            host(r'api', 'urls.api', name='api'),
            host(r'admin', 'urls.admin', name='admin', scheme='https://'),
        )

    :param regex: a regular expression to be used to match the request's
                  host.
    :type regex: str
    :param urlconf: the dotted path of a URLconf module of the host
    :type urlconf: str
    :param callback: a callable or the dotted path of a callable to be used
                     when matching has happened
    :type callback: callable or str
    :param prefix: the prefix to apply to the ``urlconf`` parameter
    :type prefix: str
    :param scheme: the scheme to prepend host names with during reversing,
                   e.g. when using the host_url() template tag. Defaults to
                   :attr:`~django.conf.settings.HOST_SCHEME`.
    :param port: the port to append to host names during reversing,
                 e.g. when using the host_url() template tag. Defaults to
                 :attr:`~django.conf.settings.HOST_PORT`.
    :type scheme: str
    """
    def __init__(self, regex, urlconf, name, callback=None, prefix='',
                 scheme=None, port=None):
        """
        Compile hosts. We add a literal fullstop to the end of every
        pattern to avoid rather unwieldy escaping in every definition.
        """
        self.regex = regex
        self.compiled_regex = re.compile(r'%s(\.|$)' % regex)
        self.urlconf = urlconf
        self.name = name
        self._scheme = scheme
        self._port = port
        if callable(callback):
            self._callback = callback
        else:
            self._callback, self._callback_str = None, callback
        self.add_prefix(prefix)

    def __repr__(self):
        return smart_str('<%s %s: regex=%r urlconf=%r scheme=%r port=%r>' %
                         (self.__class__.__name__, self.name, self.regex,
                          self.urlconf, self.scheme, self.port))

    @cached_property
    def scheme(self):
        if self._scheme is None:
            self._scheme = getattr(settings, 'HOST_SCHEME', '//')
        return normalize_scheme(self._scheme)

    @cached_property
    def port(self):
        if self._port is None:
            self._port = getattr(settings, 'HOST_PORT', '')
        return normalize_port(self._port)

    @property
    def callback(self):
        if self._callback is not None:
            return self._callback
        elif self._callback_str is None:
            return lambda *args, **kwargs: None
        try:
            self._callback = get_callable(self._callback_str)
        except ImportError as exc:
            mod_name, _ = get_mod_func(self._callback_str)
            raise ImproperlyConfigured("Could not import '%s'. "
                                       "Error was: %s" %
                                       (mod_name, str(exc)))
        except AttributeError as exc:
            mod_name, func_name = get_mod_func(self._callback_str)
            raise ImproperlyConfigured("Tried importing '%s' from module "
                                       "'%s' but failed. Error was: %s" %
                                       (func_name, mod_name, str(exc)))
        return self._callback

    def add_prefix(self, prefix=''):
        """
        Adds the prefix string to a string-based urlconf.
        """
        if prefix:
            self.urlconf = prefix.rstrip('.') + '.' + self.urlconf



from django.conf import settings
from django.db import models
from django.db.models.fields import FieldDoesNotExist


class HostSiteManager(models.Manager):
    """
    A model manager to limit objects to those associated with a site.

    :param field_name: the name of the related field pointing at the
                       :class:`~django.contrib.sites.models.Site` model,
                       or a series of relations using the
                       ``field1__field2__field3`` notation. Falls back
                       to looking for 'site' and 'sites' fields.
    :param select_related: a boolean specifying whether to use
                           :meth:`~django.db.models.QuerySet.select_related`
                           when querying the database

    Define a manager instance in your model class with one
    of the following notations::

        on_site = HostSiteManager()  # automatically looks for site and sites
        on_site = HostSiteManager("author__site")
        on_site = HostSiteManager("author__blog__site")
        on_site = HostSiteManager("author__blog__site",
                                  select_related=False)

    Then query against it with one of the manager methods::

        def home_page(request):
            posts = BlogPost.on_site.by_request(request).all()
            return render(request, 'home_page.html', {'posts': posts})

    """
    def __init__(self, field_name=None, select_related=True):
        super(HostSiteManager, self).__init__()
        self._field_name = field_name
        self._select_related = select_related
        self._depth = 1
        self._is_validated = False

    def _validate_field_name(self):
        field_names = self.model._meta.get_all_field_names()

        # If a custom name is provided, make sure the field exists on the model
        if self._field_name is not None:
            name_parts = self._field_name.split("__", 1)
            rel_depth = len(name_parts)
            if rel_depth > self._depth:
                self._depth = rel_depth
            field_name = name_parts[0]
            if field_name not in field_names:
                raise ValueError("%s couldn't find a field named %s in %s." %
                                 (self.__class__.__name__, field_name,
                                  self.model._meta.object_name))

        # Otherwise, see if there is a field called either 'site' or 'sites'
        else:
            for potential_name in ['site', 'sites']:
                if potential_name in field_names:
                    self._field_name = field_name = potential_name
                    self._is_validated = True
                    break
                else:
                    field_name = None

        # Now do a type check on the field (FK or M2M only)
        try:
            field = self.model._meta.get_field(field_name)
            if not isinstance(field, (models.ForeignKey,
                                      models.ManyToManyField)):
                raise TypeError("%s must be a ForeignKey or "
                                "ManyToManyField." % field_name)
        except FieldDoesNotExist:
            raise ValueError("%s couldn't find a field named %s in %s." %
                             (self.__class__.__name__, field_name,
                              self.model._meta.object_name))
        self._is_validated = True

    def get_queryset(self, site_id=None):
        if site_id is None:
            site_id = settings.SITE_ID
        if not self._is_validated:
            self._validate_field_name()
        qs = super(HostSiteManager, self).get_queryset()
        return qs.filter(**{'%s__id__exact' % self._field_name: site_id})

    def by_id(self, site_id=None):
        """
        Returns a queryset matching the given site id. If not given
        this falls back to the ``SITE_ID`` setting.

        :param site_id: the ID of the site
        :rtype: :class:`~django.db.models.query.QuerySet`
        """
        return self.get_queryset(site_id)

    def by_request(self, request):
        """
        Returns a queryset matching the given request's site
        attribute.

        :param request: the current request
        :type request: :class:`~django.http.HttpRequest`
        :rtype: :class:`~django.db.models.query.QuerySet`
        """
        if not hasattr(request, "site") or request.site is None:
            return self.none()
        return self.by_site(request.site)

    def by_site(self, site):
        """
        Returns a queryset matching the given site.

        :param site: a site instance
        :type site: :class:`~django.contrib.sites.models.Site`
        :rtype: :class:`~django.db.models.query.QuerySet`
        """
        return self.by_id(site.id)



from django.template import Context, Template, TemplateSyntaxError
from django.template.base import Parser
from django.test.utils import override_settings
from django_hosts.templatetags.hosts import parse_params

from .base import HostsTestCase


class TemplateTagsTest(HostsTestCase):

    def render(self, template, context=None):
        if context is None:
            context = Context({})
        return Template('{% load hosts %}' + template).render(context)

    def assertRender(self, template, expected, context=None):
        rendered = self.render(template, context)
        self.assertEqual(expected, rendered.strip())

    @override_settings(
        DEFAULT_HOST='www',
        ROOT_HOSTCONF='tests.hosts.simple')
    def test_host_url_tag_simple(self):
        self.assertRender("{% host_url 'simple-direct' host 'www' %}",
                          '//www.example.com/simple/')
        self.assertRender("{% host_url 'simple-direct' host 'www' as "
                          "simple_direct_url %}{{ simple_direct_url }}",
                          '//www.example.com/simple/')

    @override_settings(
        DEFAULT_HOST='www',
        ROOT_HOSTCONF='tests.hosts.simple')
    def test_url_tag_override(self):
        # we should be setting HOST_OVERRIDE_URL_TAG to True
        # but that doesn't really work since that setting is read only
        # on setup time so we have to fake it by manually setting the stage.
        try:
            from django.template.base import add_to_builtins
        except ImportError:  # Django 1.9+
            add_to_builtins = None

        if add_to_builtins:
            add_to_builtins('django_hosts.templatetags.hosts_override')

            self.assertRender("{% url 'simple-direct' host 'www' %}",
                          '//www.example.com/simple/')
            self.assertRender("{% url 'simple-direct' host 'www' as "
                              "simple_direct_url %}{{ simple_direct_url }}",
                              '//www.example.com/simple/')
        else:
            with self.settings(
                TEMPLATES=[{
                    'BACKEND': 'django.template.backends.django.DjangoTemplates',
                    'OPTIONS': {
                        'builtins': [
                            'django_hosts.templatetags.hosts_override',
                        ],
                    },
                }]
            ):
                self.assertRender("{% url 'simple-direct' host 'www' %}",
                                  '//www.example.com/simple/')
                self.assertRender("{% url 'simple-direct' host 'www' as "
                                  "simple_direct_url %}{{ simple_direct_url }}",
                                  '//www.example.com/simple/')

    @override_settings(
        DEFAULT_HOST='www',
        ROOT_HOSTCONF='tests.hosts.simple')
    def test_host_url_tag_without_host(self):
        self.assertRender("{% host_url 'simple-direct' %}",
                          '//www.example.com/simple/')

    @override_settings(
        DEFAULT_HOST='www',
        ROOT_HOSTCONF='tests.hosts.simple')
    def test_host_url_tag_with_scheme(self):
        self.assertRender("{% host_url 'simple-direct' scheme 'http' %}",
                          'http://www.example.com/simple/')
        self.assertRender("{% host_url 'simple-direct' scheme 'git' %}",
                          'git://www.example.com/simple/')

    @override_settings(
        DEFAULT_HOST='port-tag',
        ROOT_HOSTCONF='tests.hosts.simple',
        HOST_SCHEME='http',
        PARENT_HOST='example.com')
    def test_host_url_tag_with_port(self):
        self.assertRender("{% host_url 'simple-direct' port '8000' %}",
                          'http://port-tag.example.com:8000/simple/')

    @override_settings(
        DEFAULT_HOST='www',
        ROOT_HOSTCONF='tests.hosts.simple')
    def test_host_url_tag_with_args(self):
        self.assertRender(
            "{% host_url 'simple-direct' host 'with_args' 'www.eggs.spam' %}",
            '//www.eggs.spam/simple/')
        self.assertRender("{% host_url 'simple-direct' as yeah "
                          "host 'with_args' 'www.eggs.spam' %}{{ yeah }}",
                          '//www.eggs.spam/simple/')

    @override_settings(
        DEFAULT_HOST='www',
        PARENT_HOST='eggs.spam',
        ROOT_HOSTCONF='tests.hosts.simple')
    def test_host_url_tag_with_kwargs(self):
        self.assertRender(
            "{% host_url 'simple-direct' "
            "host 'with_kwargs' username='johndoe' %}",
            '//johndoe.eggs.spam/simple/')

    @override_settings(
        DEFAULT_HOST='www',
        PARENT_HOST='eggs.spam',
        ROOT_HOSTCONF='tests.hosts.simple')
    def test_host_url_tag_with_view_kwargs(self):
        self.assertRender(
            "{% host_url 'complex-direct' template='test' "
            "host 'with_view_kwargs' subdomain='test2000' %}",
            '//stest2000.eggs.spam/template/test/')

    @override_settings(
        DEFAULT_HOST='www',
        ROOT_HOSTCONF='tests.hosts.simple',
        PARENT_HOST='eggs.spam')
    def test_host_url_tag_parent_host(self):
        self.assertRender("{% host_url 'simple-direct' host 'static' %}",
                          '//static.eggs.spam/simple/')

    @override_settings(
        DEFAULT_HOST='without_www',
        ROOT_HOSTCONF='tests.hosts.simple',
        PARENT_HOST='example.com')
    def test_host_url_no_www(self):
        self.assertRender("{% host_url 'simple-direct' host 'without_www' %}",
                          '//example.com/simple/')

    @override_settings(
        DEFAULT_HOST='www',
        ROOT_HOSTCONF='tests.hosts.simple')
    def test_raises_template_syntaxerror(self):
        self.assertRaises(TemplateSyntaxError,
                          self.render, "{% host_url %}")
        self.assertRaises(TemplateSyntaxError, self.render,
                          "{% host_url 'simple-direct' host %}")
        self.assertRaises(TemplateSyntaxError, self.render,
                          "{% host_url 'simple-direct' as %}")
        self.assertRaises(TemplateSyntaxError, self.render,
                          "{% host_url simple-direct %}")
        self.assertRaises(TemplateSyntaxError, parse_params,
                          'host_url', Parser(['']), "username=='johndoe'")
        self.assertRaises(TemplateSyntaxError, parse_params,
                          'host_url', Parser(['']), "\n='johndoe'")



"""
If you want to reverse the hostname or the full URL or a view including the
scheme, hostname and port you'll need to use the ``reverse`` and
``reverse_host`` helper functions (or its lazy cousins).
"""
import re

from importlib import import_module

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.core.signals import setting_changed
from django.core.urlresolvers import NoReverseMatch, reverse as reverse_path
from django.utils import six
from django.utils.encoding import iri_to_uri, force_text
from django.utils.functional import lazy
from django.utils.lru_cache import lru_cache
from django.utils.regex_helper import normalize

from .defaults import host as host_cls
from .utils import normalize_scheme, normalize_port


@lru_cache()
def get_hostconf():
    try:
        return settings.ROOT_HOSTCONF
    except AttributeError:
        raise ImproperlyConfigured("Missing ROOT_HOSTCONF setting")


@lru_cache()
def get_hostconf_module(hostconf=None):
    if hostconf is None:
        hostconf = get_hostconf()
    return import_module(hostconf)


@lru_cache()
def get_host(name=None):
    if name is None:
        try:
            name = settings.DEFAULT_HOST
        except AttributeError:
            raise ImproperlyConfigured("Missing DEFAULT_HOST setting")
    for host in get_host_patterns():
        if host.name == name:
            return host
    raise NoReverseMatch("No host called '%s' exists" % name)


@lru_cache()
def get_host_patterns():
    hostconf = get_hostconf()
    module = get_hostconf_module(hostconf)
    try:
        return module.host_patterns
    except AttributeError:
        raise ImproperlyConfigured("Missing host_patterns in '%s'" % hostconf)


def clear_host_caches():
    get_hostconf.cache_clear()
    get_hostconf_module.cache_clear()
    get_host.cache_clear()
    get_host_patterns.cache_clear()


def setting_changed_receiver(setting, enter, **kwargs):
    if not enter and setting in {'ROOT_HOSTCONF', 'DEFAULT_HOST'}:
        clear_host_caches()

setting_changed.connect(setting_changed_receiver)


def reverse_host(host, args=None, kwargs=None):
    """
    Given the host name and the appropriate parameters,
    reverses the host, e.g.::

        >>> from django.conf import settings
        >>> settings.ROOT_HOSTCONF = 'mysite.hosts'
        >>> settings.PARENT_HOST = 'example.com'
        >>> from django_hosts.resolvers import reverse_host
        >>> reverse_host('with_username', args=('jezdez',))
        'jezdez.example.com'

    :param name: the name of the host as specified in the hostconf
    :param args: the host arguments to use to find a matching entry in the
                 hostconf
    :param kwargs: similar to args but key value arguments
    :raises django.core.urlresolvers.NoReverseMatch: if no host matches
    :rtype: reversed hostname
    """
    if args and kwargs:
        raise ValueError("Don't mix *args and **kwargs in call to reverse()!")

    args = args or ()
    kwargs = kwargs or {}

    if not isinstance(host, host_cls):
        host = get_host(host)

    unicode_args = [force_text(x) for x in args]
    unicode_kwargs = dict(((k, force_text(v))
                          for (k, v) in six.iteritems(kwargs)))

    for result, params in normalize(host.regex):
        if args:
            if len(args) != len(params):
                continue
            candidate = result % dict(zip(params, unicode_args))
        else:
            if set(kwargs.keys()) != set(params):
                continue
            candidate = result % unicode_kwargs

        if re.match(host.regex, candidate, re.UNICODE):  # pragma: no cover
            parent_host = getattr(settings, 'PARENT_HOST', '').lstrip('.')
            if parent_host:
                # only add the parent host when needed (aka www-less domain)
                if candidate and candidate != parent_host:
                    candidate = '%s.%s' % (candidate, parent_host)
                else:
                    candidate = parent_host
            return candidate

    raise NoReverseMatch("Reverse host for '%s' with arguments '%s' "
                         "and keyword arguments '%s' not found." %
                         (host.name, args, kwargs))

#: The lazy version of the :func:`~django_hosts.resolvers.reverse_host`
#: function to be used in class based views and other module level situations
reverse_host_lazy = lazy(reverse_host, str)


def reverse(viewname, args=None, kwargs=None, prefix=None, current_app=None,
            host=None, host_args=None, host_kwargs=None,
            scheme=None, port=None):
    """
    Given the host and view name and the appropriate parameters,
    reverses the fully qualified URL, e.g.::

        >>> from django.conf import settings
        >>> settings.ROOT_HOSTCONF = 'mysite.hosts'
        >>> settings.PARENT_HOST = 'example.com'
        >>> from django_hosts.resolvers import reverse
        >>> reverse('about')
        '//www.example.com/about/'
        >>> reverse('about', host='www')
        '//www.example.com/about/'
        >>> reverse('repo', args=('jezdez',), host='www', scheme='git', port=1337)
        'git://jezdez.example.com:1337/repo/'

    You can set the used port and scheme in the host object or override with
    the paramater named accordingly.

    The host name can be left empty to automatically fall back to the default
    hostname as defined in the :attr:`~django.conf.settings.DEFAULT_HOST`
    setting.

    :param viewname: the name of the view
    :param args: the arguments of the view
    :param kwargs: the keyed arguments of the view
    :param prefix: the prefix of the view urlconf
    :param current_app: the current_app argument
    :param scheme: the scheme to use
    :param port: the port to use
    :param host: the name of the host
    :param host_args: the host arguments
    :param host_kwargs: the host keyed arguments
    :raises django.core.urlresolvers.NoReverseMatch: if no host or path matches
    :rtype: the fully qualified URL with path
    """
    host = get_host(host)
    hostname = reverse_host(host,
                            args=host_args,
                            kwargs=host_kwargs)
    kwargs = {
        'urlconf': host.urlconf,
        'args': args or (),
        'kwargs': kwargs or {},
        'prefix': prefix,
        'current_app': current_app,
    }
    try:
        path = reverse_path(viewname, **kwargs)
    except TypeError:
        # Django 1.9+ has the `prefix` kwarg removed in reverse().
        del kwargs['prefix']
        path = reverse_path(viewname, **kwargs)

    if scheme is None:
        scheme = host.scheme
    else:
        scheme = normalize_scheme(scheme)

    if port is None:
        port = host.port
    else:
        port = normalize_port(port)

    return iri_to_uri('%s%s%s%s' % (scheme, hostname, port, path))

#: The lazy version of the :func:`~django_hosts.resolvers.reverse`
#: function to be used in class based views and other module level situations
reverse_lazy = lazy(reverse, str)



DATABASE_ENGINE = 'sqlite3'

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}

INSTALLED_APPS = [
    'django.contrib.contenttypes',
    'django.contrib.sites',
    'django_hosts',
    'tests',
]

ROOT_URLCONF = 'tests.urls.root'

SITE_ID = 1

SECRET_KEY = 'something-something'

MIDDLEWARE_CLASSES = []



from django_hosts import patterns, host

host_patterns = patterns('',
    host(r'example\.com', 'tests.urls.simple',
         name='without_www'),
    host(r'www\.example\.com', 'tests.urls.simple', name='www'),
    host(r'static', 'tests.urls.simple', name='static'),
    host(r'^s(?P<subdomain>\w+)', 'tests.urls.complex',
         name='with_view_kwargs'),
    host(r'wiki\.(?P<domain>\w+)', 'tests.urls.simple',
         callback='django_hosts.callbacks.host_site', name='with_callback'),
    host(r'admin\.(?P<domain>\w+)', 'tests.urls.simple',
         callback='django_hosts.callbacks.cached_host_site',
         name='with_cached_callback'),
    host(r'(?P<username>\w+)', 'tests.urls.simple',
         name='with_kwargs'),
    host(r'(\w+)', 'tests.urls.simple', name='with_args'),
    host(r'scheme', 'tests.urls.simple', name='scheme',
         scheme='https://'),
    host(r'port', 'tests.urls.simple', name='port',
         port='12345'),
    host(r'port-tag', 'tests.urls.simple', name='port-tag',
         port='12345'),
)



import pytest
from django.test import TestCase, RequestFactory


@pytest.mark.django_db()
class HostsTestCase(TestCase):

    def setUp(self):
        super(HostsTestCase, self).setUp()
        # Every test needs access to the request factory.
        self.factory = RequestFactory()

    def assertRaisesMessageIn(self, error,
                              message, callable, *args, **kwargs):
        self.assertRaises(error, callable, *args, **kwargs)
        try:
            callable(*args, **kwargs)
        except error as exc:
            self.assertIn(message, str(exc))



# flake8: noqa
import pkg_resources

try:  # pragma: no cover
    from django_hosts.defaults import patterns, host
    from django_hosts.resolvers import (reverse, reverse_lazy,
                                        reverse_host, reverse_host_lazy)
except ImportError:  # pragma: no cover
    pass

__version__ = pkg_resources.get_distribution('django-hosts').version
__author__ = 'Jannis Leidel <jannis@leidel.info>'

default_app_config = 'django_hosts.apps.HostsConfig'



import codecs
from os import path
from setuptools import setup, find_packages


def read(*parts):
    filename = path.join(path.dirname(__file__), *parts)
    with codecs.open(filename, encoding='utf-8') as fp:
        return fp.read()

setup(
    name='django-hosts',
    description='Dynamic and static host resolving for Django. '
                'Maps hostnames to URLconfs.',
    long_description=read('README.rst'),
    use_scm_version=True,
    setup_requires=['setuptools_scm'],
    url='http://django-hosts.rtfd.org/',
    author='Jannis Leidel',
    author_email='jannis@leidel.info',
    license='BSD',
    packages=find_packages(),
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.8',
        'Framework :: Django :: 1.9',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],
)



from django.contrib.sites.models import Site
from django.test import RequestFactory
from django.test.utils import override_settings
from django.utils.functional import empty

from django_hosts.middleware import HostsRequestMiddleware

from .base import HostsTestCase
from .models import Author, BlogPost, WikiPage


class SitesTests(HostsTestCase):

    def setUp(self):
        super(SitesTests, self).setUp()
        self.site1 = Site.objects.create(domain='wiki.site1', name='site1')
        self.site2 = Site.objects.create(domain='wiki.site2', name='site2')
        self.site3 = Site.objects.create(domain='wiki.site3', name='site3')
        self.site4 = Site.objects.create(domain='admin.site4', name='site4')
        self.page1 = WikiPage.objects.create(content='page1', site=self.site1)
        self.page2 = WikiPage.objects.create(content='page2', site=self.site1)
        self.page3 = WikiPage.objects.create(content='page3', site=self.site2)
        self.page4 = WikiPage.objects.create(content='page4', site=self.site3)

        self.author1 = Author.objects.create(name='john', site=self.site1)
        self.author2 = Author.objects.create(name='terry', site=self.site2)
        self.post1 = BlogPost.objects.create(content='post1',
                                             author=self.author1)
        self.post2 = BlogPost.objects.create(content='post2',
                                             author=self.author2)

    def tearDown(self):
        for model in [WikiPage, BlogPost, Author, Site]:
            model.objects.all().delete()

    @override_settings(
        ROOT_HOSTCONF='tests.hosts.simple',
        DEFAULT_HOST='www')
    def test_sites_callback(self):
        rf = RequestFactory(HTTP_HOST='wiki.site1')
        request = rf.get('/simple/')
        middleware = HostsRequestMiddleware()
        middleware.process_request(request)
        self.assertEqual(request.urlconf, 'tests.urls.simple')
        self.assertEqual(request.site.pk, self.site1.pk)

    @override_settings(
        ROOT_HOSTCONF='tests.hosts.simple',
        DEFAULT_HOST='www')
    def test_sites_cached_callback(self):
        rf = RequestFactory(HTTP_HOST='admin.site4')
        request = rf.get('/simple/')
        middleware = HostsRequestMiddleware()
        middleware.process_request(request)

        get_site = lambda: request.site.domain

        # first checking if there is a db query
        self.assertEqual(request.site._wrapped, empty)
        self.assertNumQueries(1, get_site)
        self.assertEqual(request.site._wrapped, self.site4)

        # resetting the wrapped site instance to check the cache value
        request.site._wrapped = empty
        self.assertNumQueries(0, get_site)
        self.assertEqual(request.site.pk, self.site4.pk)

    @override_settings(
        ROOT_HOSTCONF='tests.hosts.simple',
        DEFAULT_HOST='www')
    def test_sites_callback_with_parent_host(self):
        rf = RequestFactory(HTTP_HOST='wiki.site2')
        request = rf.get('/simple/')
        middleware = HostsRequestMiddleware()
        middleware.process_request(request)
        self.assertEqual(request.urlconf, 'tests.urls.simple')
        self.assertEqual(request.site.pk, self.site2.pk)

    @override_settings(
        ROOT_HOSTCONF='tests.hosts.simple',
        DEFAULT_HOST='www')
    def test_manager_simple(self):
        rf = RequestFactory(HTTP_HOST='wiki.site2')
        request = rf.get('/simple/')
        middleware = HostsRequestMiddleware()
        middleware.process_request(request)
        self.assertEqual(request.urlconf, 'tests.urls.simple')
        self.assertEqual(request.site.pk, self.site2.pk)
        self.assertEqual(list(WikiPage.on_site.by_request(request)),
                         [self.page3])

    @override_settings(
        ROOT_HOSTCONF='tests.hosts.simple',
        DEFAULT_HOST='www')
    def test_manager_missing_site(self):
        rf = RequestFactory(HTTP_HOST='static')
        request = rf.get('/simple/')
        middleware = HostsRequestMiddleware()
        middleware.process_request(request)
        self.assertEqual(request.urlconf, 'tests.urls.simple')
        self.assertRaises(AttributeError, lambda: request.site)
        self.assertEqual(list(WikiPage.on_site.by_request(request)), [])

    def test_manager_default_site(self):
        with self.settings(SITE_ID=self.site1.id):
            self.assertEqual(list(WikiPage.on_site.all()),
                             [self.page1, self.page2])

    def test_manager_related_site(self):
        with self.settings(SITE_ID=self.site1.id):
            self.assertEqual(list(BlogPost.on_site.all()), [self.post1])
        with self.settings(SITE_ID=self.site2.id):
            self.assertEqual(list(BlogPost.on_site.all()), [self.post2])

    def test_no_select_related(self):
        with self.settings(SITE_ID=self.site1.id):
            self.assertEqual(list(BlogPost.no_select_related.all()),
                             [self.post1])

    def test_non_existing_field(self):
        with self.settings(SITE_ID=self.site1.id):
            self.assertRaises(ValueError, BlogPost.non_existing.all)

    def test_dead_end_field(self):
        with self.settings(SITE_ID=self.site1.id):
            self.assertRaises(ValueError, BlogPost.dead_end.all)

    def test_non_rel_field(self):
        with self.settings(SITE_ID=self.site1.id):
            self.assertRaises(TypeError, BlogPost.non_rel.all)



from django.db import models
from django.contrib.sites.models import Site

from django_hosts.managers import HostSiteManager


class Author(models.Model):
    name = models.TextField()
    site = models.ForeignKey(Site)


class BlogPost(models.Model):
    content = models.TextField()
    author = models.ForeignKey(Author)

    objects = models.Manager()
    dead_end = HostSiteManager()
    on_site = HostSiteManager('author__site')
    no_select_related = HostSiteManager('author__site',
                                        select_related=False)
    non_existing = HostSiteManager('blabla')
    non_rel = HostSiteManager('content')

    def __unicode__(self):
        return str(self.id)


class WikiPage(models.Model):
    content = models.TextField()
    site = models.ForeignKey(Site)

    objects = models.Manager()
    on_site = HostSiteManager()

    def __unicode__(self):
        return str(self.id)
